#include <stdio.h>
#include <string.h>
#include <sys/unistd.h>
#include <sys/stat.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_log.h"
#include "esp_system.h"
#include "esp_event.h"
#include "esp_event_loop.h"
#include "nvs_flash.h"
#include "driver/gpio.h"
#include "esp_err.h"
#include "esp_vfs_fat.h"
#include "driver/sdmmc_host.h"
#include "driver/sdmmc_defs.h"
#include "sdmmc_cmd.h"


esp_err_t event_handler(void *ctx, system_event_t *event)
{
    return ESP_OK;
}

static const char* TAG = "example";

#define FILE_BUFFER_SIZE 128

void test_sd(void*none)
{
    char * name = "/sdcard/foo.txt";
    FILE * f;
    ESP_LOGI(TAG, "tried to open %s", name);
    ESP_LOGI(TAG, "strlen: %d", strlen(name));
    f = fopen(name, "r");
    if (f == 0) {
        ESP_LOGI(TAG, "couldn't fopen(%s): %d", name, errno);
    }
    ESP_LOGI(TAG, "opened: %d", fileno(f));
    char buf[FILE_BUFFER_SIZE];
    int ct = 0;
    ct = fread(buf, sizeof(char), FILE_BUFFER_SIZE, f);
    ESP_LOGI(TAG, "read: %d bytes", ct);
    ESP_LOGI(TAG, "read: %s", buf);
    fclose(f);
}

void app_main(void)
{
    nvs_flash_init();

    ESP_LOGI(TAG, "Initializing SD card");

    sdmmc_host_t host = SDMMC_HOST_DEFAULT();

    // To use 1-line SD mode, uncomment the following line:
    // host.flags = SDMMC_HOST_FLAG_1BIT;

    // This initializes the slot without card detect (CD) and write protect (WP) signals.
    // Modify slot_config.gpio_cd and slot_config.gpio_wp if your board has these signals.
    sdmmc_slot_config_t slot_config = SDMMC_SLOT_CONFIG_DEFAULT();

    // Options for mounting the filesystem.
    // If format_if_mount_failed is set to true, SD card will be partitioned and formatted
    // in case when mounting fails.
    esp_vfs_fat_sdmmc_mount_config_t mount_config = {
        .format_if_mount_failed = false,
        .max_files = 5
    };

    // Use settings defined above to initialize SD card and mount FAT filesystem.
    // Note: esp_vfs_fat_sdmmc_mount is an all-in-one convenience function.
    // Please check its source code and implement error recovery when developing
    // production applications.
    sdmmc_card_t* card;
    esp_err_t ret = esp_vfs_fat_sdmmc_mount("/sdcard", &host, &slot_config, &mount_config, &card);
    if (ret != ESP_OK) {
        if (ret == ESP_FAIL) {
            ESP_LOGE(TAG, "Failed to mount filesystem. If you want the card to be formatted, set format_if_mount_failed = true.");
        } else {
            ESP_LOGE(TAG, "Failed to initialize the card (%d). Make sure SD card lines have pull-up resistors in place.", ret);
        }
        return;
    }

    // Card has been initialized, print its properties
    sdmmc_card_print_info(stdout, card);

//SUCCEEDS
    test_sd(NULL);

//SUCCEEDS
    test_sd(NULL);

//FAILS
    xTaskCreate(&test_sd, "test_sd", 8192, NULL, 5, NULL);

    gpio_set_direction(GPIO_NUM_4, GPIO_MODE_OUTPUT);
    int level = 0;
    while (true) {
        gpio_set_level(GPIO_NUM_4, level);
        level = !level;
        vTaskDelay(300 / portTICK_PERIOD_MS);
    }

    // All done, unmount partition and disable SDMMC host peripheral
    esp_vfs_fat_sdmmc_unmount();
    ESP_LOGI(TAG, "Card unmounted");
}
